<?php
require_once 'vendor/autoload.php';
require_once('crest/crest.php');
require_once('handlersettings.php');

use Kibra\IblockList;
use Kibra\CrmList;

class Back
{
	public static function debug($data)
	{
		echo "<pre>";
		echo print_r($data,true);
		echo "</pre>";
	}

    public static function save($params)
    {
        $id = $params['id'];

        $company = $params['company'];
        $driver = $params['driver'];
        $vehicle = $params['vehicle'];
        $trailer = $params['trailer'];
        $result = CRest::call(
            'crm.item.update',
            [
                'entityTypeId' => ENTITY_TYPE_ID,
                'id' => $id,
                'fields' => [
                    FIELD_COMPANY => $company,
                    FIELD_DRIVER => $driver,
                    FIELD_VEHICLE => $vehicle,
                    FIELD_TRAILER => $trailer,
                ]
            ]
        );
        if(isset($result['error'])) {
            return false;
        } else {
            return true;
        }        
    }

    public static function getVehicleAndTrailers($params)
    {
        $vehicles = [];
        $trailers = [];
        $companyId = $params['companyId'];

        $paramsVehicles = [
            'iblockId' => VEHICLES_LIST_ID,
            'filter' => [
                VEHICLES_LIST_COMPANY_FIELD => $companyId,
            ],
        ];
        $listVehicles = new IblockList('CRest', $paramsVehicles);
        $listVehiclesResult = $listVehicles->getList();
        if(!$listVehiclesResult['error']) {
            foreach($listVehiclesResult['result'] as $vehicle) {
                $vehicles[] = [
                    'id' => $vehicle['ID'],
                    'value' => $vehicle[VEHICLES_LIST_VALUE_FIELD]
                ];
            }
        }

        $paramsTrailers = [
            'iblockId' => TRAILERS_LIST_ID,
            'filter' => [
                TRAILERS_LIST_COMPANY_FIELD => $companyId,
            ],
        ];
        $listTrailers = new IblockList('CRest', $paramsTrailers);
        $listTrailersResult = $listTrailers->getList();
        if(!$listTrailersResult['error']) {
            foreach($listTrailersResult['result'] as $trailer) {
                $trailers[] = [
                    'id' => $trailer['ID'],
                    'value' => $trailer[TRAILERS_LIST_VALUE_FIELD]
                ];
            }
        }        

        $result = [
            'vehicles' => $vehicles,
            'trailers' => $trailers
        ];
        return $result;
    }

    public static function getCompanysContacts($params)
    {
        $companyId = $params['companyId'];
        $drivers = [];
        $companyContacts = CRest::call(
            'crm.company.contact.items.get',
            [
                'id' => $companyId
            ]
        );
        if((!isset($companyContacts['error'])) && (count($companyContacts['result'])>0)) {
            $filter = [];
            foreach($companyContacts['result'] as $contact) {
                $filter[] = $contact['CONTACT_ID'];
            };
            $contacts = CRest::call(
                'crm.contact.list',
                [
                    'filter' => ['ID' => $filter],
                ]
            );
            if(!isset($contacts['error'])) {
                foreach($contacts['result'] as $contact) {
                    $lastName = $contact['LAST_NAME'] ? $contact['LAST_NAME'] : '';
                    $name = $contact['NAME'] ? $contact['NAME'] : '';
                    $secondName = $contact['SECOND_NAME'] ? $contact['SECOND_NAME'] : '';
                    $drivers[] = [
                        'id' => $contact['ID'],
                        'value' => trim($lastName . ' ' . $name . ' ' . $secondName),
                    ];
                };
            }
        }        
        return $drivers;
    }

    public static function getCompany($params)
    {
        $company = ['id' => '', 'title' => ''];
        $companyId = $params['companyId'];
        $result = CRest::call(
            'crm.company.get',
            [
                'id' => $companyId,
            ]
        );
        if(!$result['error']) {
            $company['id'] = $result['result']['ID'];
            $company['title'] = $result['result']['TITLE'];
        }
        return $company;
    }

    public static function getCompanyList()
    {
        $params = [
            'select' => ['TITLE'],
            'order' => ['TITLE' => 'ASC'],
        ];
        $c = new CrmList('CRest', 'company', $params);
        $result = $c->getList();
        if(!$result['error']) {
            foreach($result['result'] as $company) {
                $companies[] = [
                    'id' => $company['ID'],
                    'value' => $company['TITLE']
                ];
            };
        }
        return $companies;
    }

    public static function init($params)
    {
        $entityId = $params['entityId'];
        $result = [];
        $request = CRest::call(
            'crm.item.get',
            [
                'entityTypeId' => ENTITY_TYPE_ID,
                'id' => $entityId
            ]
        );
        file_put_contents('init.log', print_r($request,true));
        if(isset($request['error'])) {
            return false;
        } else {
            $element = $request['result']['item'];
            $result = [
                'company' => $element[FIELD_COMPANY] ? $element[FIELD_COMPANY] : null,
                'driver' => $element[FIELD_DRIVER] ? $element[FIELD_DRIVER] : null,
                'vehicle' => $element[FIELD_VEHICLE] ? $element[FIELD_VEHICLE] : null,
                'trailer' => $element[FIELD_TRAILER] ? $element[FIELD_TRAILER] : null,
            ];
            return $result;
        }
    }
}

$request=json_decode(file_get_contents('php://input'),true);
$params = $request['params'];

switch ($request['method']) {
	case "getCompanyList":
		echo json_encode(Back::getCompanyList());
	break;
	case "getCompany":
		echo json_encode(Back::getCompany($params));
	break;
	case "getCompanysContacts":
		echo json_encode(Back::getCompanysContacts($params));
	break;
	case "getVehicleAndTrailers":
		echo json_encode(Back::getVehicleAndTrailers($params));
	break;
	case "save":
		echo json_encode(Back::save($params));
	break;
	case "init":
		echo json_encode(Back::init($params));
	break;
};