import Select from './Select.js';
import SelectCompany from './SelectCompany.js';

class App {

    async request(method, params=[]) {
        const body={'method': method, 'params': params};
        const response = await fetch ('back.php', {
            method: 'POST',
            headers: {'Content-Type': 'application/json', 'mi': this.mi},
            body: JSON.stringify(body)
        });
        return await response.json();
    }

    getDrivers(companyId) {
        this.request('getCompanysContacts', {'companyId': companyId})
            .then(drivers => {
                this.selectDriver.addList(drivers, this.initData.driver);
            })
        this.request('getVehicleAndTrailers', {'companyId': companyId})
            .then(result => {
                this.selectVehicle.addList(result.vehicles, this.initData.vehicle);
                this.selectTrailer.addList(result.trailers, this.initData.trailer);
            })
    }

    changeCompany(companyId, parent) {
        parent.initData = {
            'company': null,
            'driver': null,
            'vehicle': null,
            'trailer': null,
        };
        parent.buttonSave.classList.remove('is-hidden');
        parent.getDrivers(companyId);
    }

    save() {
        let params = {
            'id': this.entityId,
            'company': this.selectCompany.value ? this.selectCompany.value : '',
            'driver': this.selectDriver.value ? this.selectDriver.value : '',
            'vehicle': this.selectVehicle.value ? this.selectVehicle.value : '',
            'trailer': this.selectTrailer.value ? this.selectTrailer.value : ''
        };
        console.log(params);
        this.buttonSave.classList.add('is-loading');
        this.errorMessage.classList.add('is-hidden');
        this.request('save', params)
            .then(result => {
                if(result) {
                    this.buttonSave.classList.remove('is-loading');
                    this.buttonSave.classList.add('is-hidden');
                } else {
                    this.errorMessage.classList.remove('is-hidden');
                    this.buttonSave.classList.remove('is-loading');                    
                }
            })
    }

    terminator(value, parent) {
        parent.buttonSave.classList.remove('is-hidden');
        return true;
    }

    init() {
        this.request('init', {'entityId': this.entityId})
            .then(result => {
                if(result) {
                    this.initData = result;
                    this.request('getCompany', {'companyId': this.initData.company})
                        .then(companyData => {
                            this.selectCompany.setValue(companyData);
                            this.getDrivers(this.initData.company, this);
                            this.buttonSave.classList.add('is-hidden');
                        });
                }
            })
    }

    constructor(entityId) {

        this.entityId = entityId;

        this.initData = {
            'company': null,
            'driver': null,
            'vehicle': null,
            'trailer': null,
        };

        this.wrapperFields = document.querySelector('#wrapper-fields');
        this.buttonSave = document.querySelector('#button-save');
        this.buttonSave.classList.add('is-hidden');

        this.errorMessage = document.querySelector('#error-message');
        this.errorMessage.classList.add('is-hidden');

        this.selectCompany = new SelectCompany({
            'label': 'Перевозчик',
            'field': 'UF_CRM_5_1690231416',
            'handler': this.changeCompany
        }, this);

        this.selectDriver = new Select({
            'label': 'Водитель',
            'field': 'UF_CRM_5_1690231454',
            'handler': this.terminator
        }, this);

        this.selectVehicle = new Select({
            'label': 'Тягач',
            'field': 'UF_CRM_5_1690231478',
            'handler': this.terminator
        }, this);

        this.selectTrailer = new Select({
            'label': 'Прицеп',
            'field': 'UF_CRM_5_1690231492',
            'handler': this.terminator
        }, this);

        this.init();

        this.wrapperFields.appendChild(this.selectCompany.view);
        this.wrapperFields.appendChild(this.selectDriver.view);
        this.wrapperFields.appendChild(this.selectVehicle.view);
        this.wrapperFields.appendChild(this.selectTrailer.view);

        this.buttonSave.addEventListener('click', () => {
            this.save();
        });
    }
}

const url = new URL(import.meta.url); 
const entityId = JSON.parse(url.searchParams.get("entity_id"));
console.log(entityId);

const app = new App(entityId);







