<?php
require_once('crest/crest.php');

class UserList
{

    private function orderResult()
    {
        if(count($this->order)>0) {
            $sortKey = array_key_first($this->order);
            if(isset($this->resultArray[0][$sortKey])) {
                $sortMethod = $this->order[$sortKey];
                switch ($sortMethod) {
                    case 'ASC':
                        usort($this->resultArray, function($a, $b) use ($sortKey) {return ($a[$sortKey] < $b[$sortKey]) ? -1 : 1;});
                        break;
                    case 'DESC':
                        usort($this->resultArray, function($a, $b) use ($sortKey) {return ($a[$sortKey] > $b[$sortKey]) ? -1 : 1;});
                        break;
                };
            } else {
                $this->error = [
                    'error' => 'Wrong sort key',
                    'error_description' => 'The sort key is not present in the selected keys. Sorting is not possible.',
                ];
            }
        }
    }

    private function compileResult(array $data) 
    {
        foreach($data as $dataItem) {
            $this->resultArray = array_merge($this->resultArray, $dataItem);
            $this->nextID = $this->resultArray[count($this->resultArray)-1]['ID'];
        }
    }

    private function batch(float $batchCount):bool
    {
        $arBatch = [];
        for($batchItem = 0; $batchItem < $batchCount; $batchItem++) {
            if ($batchItem == 0) {
                $filter = array_merge($this->filter, ["<ID" => $this->nextID]);
            } else {
                $filter = array_merge($this->filter, ['<ID' => '$result[' . strval($batchItem-1) . '][49][ID]']);
            }
            $arParams = [   
                'start' => '-1',
                'sort' => 'ID',
                'order' => 'DESC',
                'FILTER' => $filter,
            ];
            $arBatch[]=[
                'method' => $this->method,
                'params' => $arParams,
            ];
            $this->countOfBatches--;
        };
        if(count($arBatch)>0) {
            $result = $this->crest::callBatch($arBatch);
            if ($result['result']['result_error']) {
                $this->error = $result['result']['result_error'];
                return false;
            } else {
                $this->compileResult($result['result']['result']);
                return true;            
            }
        }
    }

    public function getList():array
    {
        if (count($this->error)>0) {
            return ['result' => $this->resultArray, 'total' => $this->total, 'error' => $this->error];
        } else if ($this->resultPresent) {
            $this->orderResult();
            return ['result' => $this->resultArray, 'total' => $this->total, 'error' => $this->error];
        } else {
            for($page = 0; $page < $this->countOfPages; $page++){
                if (!$this->batch(50)) return ['result' => $this->resultArray, 'total' => $this->total, 'error' => $this->error];
            }
            if (!($this->batch($this->countOfBatches))) return ['result' => $this->resultArray, 'total' => $this->total, 'error' => $this->error];   
            $this->orderResult();     
            return ['result' => $this->resultArray, 'total' => $this->total, 'error' => $this->error];
        }        
    }

    private function init()
    {
        $result = $this->crest::call(
            $this->method, 
            [
                'sort' => 'ID',
                'order' => 'DESC',
                'FILTER' => $this->filter,
            ]
        );
        if (isset($result['error'])) {
            $this->error = [
                'error' => $result['error'],
                'error_description' => isset($result['error_description']) ? $result['error_description'] : '',
                'error_information' => isset($result['error_information']) ? $result['error_information'] : '',
            ];
        } else {
            $this->total = $result['total'];
            if ($this->total<=50) {
                $this->resultArray = $result['result'];
                $this->resultPresent = true;
            } else {
                $maxID = $result['result'][0]['ID'];
                $this->nextID = $maxID + 1;
                $this->countOfBatches = ceil($this->total/50);
                $this->countOfPages = intdiv($this->total, 2500);
            }
        }
    }

    public function __construct($crest, $params) 
    {
        $this->crest = $crest;

        $this->resultPresent = false;
        $this->resultArray = [];
        $this->total = 0;
        $this->error = [];
        $this->nextID = 0;

        $this->method = 'user.get';
        $this->filter = isset($params['filter']) ? $params['filter'] : [];
        $this->order = isset($params['order']) ? $params['order'] : [];
        $this->init();
    }
}