<?php
declare(strict_types=1);

namespace Kibra;

abstract class Robot
{
    /**
     * Send result of robot/activity to B24
     *
     * @param array $response
     * @return void
     */
    protected function send_result(array $response)
    {
        return (array) $this->s_crest::call (
            'bizproc.event.send',
            [
                'EVENT_TOKEN' => $this->s_eventToken,
                'RETURN_VALUES' => $response
            ]
        );
    }

    /**
     * Send message to log
     *
     * @param string $message
     * @return void
     */
    protected function send_message(string $message)
    {
        return (array) $this->s_crest::call (
            'bizproc.activity.log',
            [
                'EVENT_TOKEN' => $this->s_eventToken,
                'LOG_MESSAGE' => $message,
            ]
        );
    }

    /**
     * Set error log filename
     *
     * @param string $name
     * @return boolean
     */
    public function setErrorLogName(string $name) {
        $this->errorLogName = $name;
        return true;
    }

    /**
     * Save error log
     *
     * @return boolean
     */
    private function errorLog()
    {
        return file_put_contents('log/' . date("Y-m-d H-i-s") . ' - error.log', print_r($this->errors,true));
    }

    /**
     * Parse properties of activity/robot (user properties only)
     *
     * @return boolean
     */
    private function parse_properties()
    {
        foreach($this->s_properties as $key => $property) {
            $this->$key = $property;
        };
        return (boolean) true;
    }

    /**
     * Entry point of class
     *
     * @return void
     */
    abstract public function run();

    /**
     *
     * @param Array $request
     * Request from B24
     * 
     * @param String $crest
     * Name of CRest class
     */
    public function __construct(array $request, string $crest)
    {
        $this->s_crest = $crest;
        $this->s_eventToken = $request['event_token'];
		$this->s_domain = $request['auth']['domain'];
        $this->s_properties = $request['properties'] ?? [];
        $this->parse_properties();

        $this->errorLogName = 'log/' . date("Y-m-d H-i-s") . ' - error.log';

        $this->s_use_subscription = $request['use_subscription'];
        $this->s_workflow_id = $request['workflow_id'];
        $this->s_code = $request['code'];
        $this->s_entity_type = $request['document_type'][2];
        $this->s_entity_id = str_replace($this->s_entity_type . '_','',$request['document_id'][2]);
    }
}
?>
